@@ -1,9 +1,12 @@
+from enum import Enum
 from datetime import datetime, date
+from decimal import Decimal
 
+import pytz
 from google.protobuf import json_format
 
 from .dateparse import *
-from .timezone import StrTzInfoType, make_aware
+from .timezone import StrTzInfoType, make_aware, is_aware
 
 
 class ProtobufParser(json_format._Parser):  # noqa
@@ -13,8 +16,6 @@ class ProtobufParser(json_format._Parser):  # noqa
             self._ConvertStructMessage(value, message.struct_value)
         elif isinstance(value, list):
             self._ConvertListValueMessage(value, message.list_value)
-        elif isinstance(value, (datetime, date)):
-            message.string_value = value.isoformat()
         elif value is None:
             message.null_value = 0
         elif isinstance(value, bool):
@@ -23,6 +24,16 @@ class ProtobufParser(json_format._Parser):  # noqa
             message.string_value = value
         elif isinstance(value, json_format._INT_OR_FLOAT):  # noqa
             message.number_value = value
+        elif isinstance(value, date):
+            message.string_value = value.isoformat()
+        elif isinstance(value, datetime):
+            if is_aware(value):
+                value = value.astimezone(pytz.utc)
+            message.string_value = value.isoformat()
+        elif isinstance(value, Enum):
+            message.string_value = value.name
+        elif isinstance(value, Decimal):
+            message.string_value = str(value)
         else:
             raise json_format.ParseError(
                 'Value {0} has unexpected type {1}.'.format(value, type(value))
@@ -54,7 +65,10 @@ class ProtobufPrinter(json_format._Printer):  # noqa
                 # Use base64 Data encoding for bytes
                 return json_format.base64.b64encode(value).decode('utf-8')
             else:
-                return value
+                try:
+                    return datetime.fromisoformat(value)
+                except ValueError:
+                    return value
         elif field.cpp_type == json_format.descriptor.FieldDescriptor.CPPTYPE_BOOL:
             return bool(value)
         elif field.cpp_type in json_format._INT64_TYPES:  # noqa
@@ -83,7 +97,7 @@ def MessageToDict(  # noqa
         preserving_proto_field_name=False,
         use_integers_for_enums=False,
         descriptor_pool=None,
-        float_precision=None
+        float_precision=None,
 ):
     printer = ProtobufPrinter(
         including_default_value_fields,
@@ -96,7 +110,10 @@ def MessageToDict(  # noqa
 
 
 def ParseDict(  # noqa
-    js_dict, message, ignore_unknown_fields=False, descriptor_pool=None
+    js_dict,
+    message,
+    ignore_unknown_fields=False,
+    descriptor_pool=None,
 ):
     parser = ProtobufParser(ignore_unknown_fields, descriptor_pool)
     parser.ConvertMessage(js_dict, message)
